package com.bomostory.sceneeditmodule

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Paint
import android.util.Log
import com.bomostory.sceneeditmodule.MovieEditActivity.Companion.FPS
import com.bomostory.sceneeditmodule.basicdata.Music
import com.bomostory.sceneeditmodule.basicdata.Scene
import com.example.exportmedia.MediaHelper
import com.example.exportmedia.data.AudioSource
import com.example.exportmedia.vedio.MovieMaker
import io.reactivex.Observable
import java.io.File
import java.util.AbstractMap
import kotlin.collections.ArrayList
import kotlin.collections.LinkedHashMap
import kotlin.collections.List
import kotlin.collections.set
import kotlin.math.pow

class SuperMovieMaker {

    fun make(
            outputFile: File,
            scene: Scene,
            musics: List<Music>,
            scaleWidth: Int,
            scaleHeight: Int,
            mediaHelper: MediaHelper): Observable<String> {
        return Observable.create<String> { emitter ->
            val movieBuilder = MovieMaker.Builder(mediaHelper)
            movieBuilder.fps = FPS
            movieBuilder.audioSources = generateAudioSource(musics)

            val inputSceneStatus = LinkedHashMap<Int, Int>()
            var sceneIndex = -1
            var sceneRepeatCount = 0
            var sceneCount = 0

            var totalFrame = 0
            scene?.record?.period?.let {
                totalFrame = (it / ((1f / FPS) * 1000).toLong()).toInt()
            }

            generateMovieFilms(
                    scene,
                    scaleWidth,
                    scaleHeight
            )
                    .subscribe {
                        val msg = "progress ${sceneCount++}/$totalFrame"
                        emitter.onNext("Compute image $msg")
                        Log.d("Debug", msg)
                        if (it.key == sceneIndex) {
                            sceneRepeatCount++
                        } else {
                            sceneIndex = it.key
                            sceneRepeatCount = 1
                            movieBuilder.addImage(it.key, it.value).subscribe()
                            emitter.onNext("Save image $msg")
                        }
                        inputSceneStatus[sceneIndex] = sceneRepeatCount
                    }

            movieBuilder.inputFilePath = movieBuilder.addInputFile(inputSceneStatus).blockingFirst()
            emitter.onNext("Encoding video")
            movieBuilder.build().output(outputFile).blockingSubscribe()
            emitter.onComplete()
        }
    }

    private fun generateAudioSource(musics: List<Music>): ArrayList<AudioSource> {
        val audioSources = ArrayList<AudioSource>()
        for (music in musics) {
            val audioSource = AudioSource()
            audioSource.duration = music.duration
            audioSource.offsetSec = music.offsetTime.toFloat() / 1000
            audioSource.path = music.path
            audioSources.add(audioSource)
        }
        return audioSources
    }

    private fun generateMovieFilms(
            scene: Scene,
            scaleWidth: Int,
            scaleHeight: Int): Observable<AbstractMap.SimpleEntry<Int, Bitmap>> {
        var preBitmap: Bitmap?
        return Observable.create<AbstractMap.SimpleEntry<Int, Bitmap>> {
            scene?.apply {
                record?.apply {
                    var bitmap = drawScene(scene, 0, scaleWidth, scaleHeight)
                    var bitmapIndex = 0
                    var trackPosition = -1

                    for (t in 0..period step ((1f / FPS) * 1000).toLong()) {
                        for (track in tracks) {
                            if (t >= track.time && tracks.indexOf(track) > trackPosition) {
                                preBitmap = bitmap
                                preBitmap?.recycle()

                                bitmap = drawScene(scene, track.positionX, scaleWidth, scaleHeight)
                                trackPosition = tracks.indexOf(track)
                            }
                        }

                        bitmap?.apply {
                            it.onNext(AbstractMap.SimpleEntry(bitmapIndex++, this))
                        }
                    }
                    it.onComplete()
                }
            }
        }
    }

    private fun drawScene(scene: Scene, trackX: Int, scaleWidth: Int, scaleHeight: Int): Bitmap? {
        var sceneBitmap: Bitmap? = null

        scene?.apply {
            sceneBitmap = BitmapFactory.decodeFile(backgroundPath)
            sceneBitmap = Bitmap.createScaledBitmap(sceneBitmap, scaleWidth, scaleHeight, true)

            val canvas = Canvas(sceneBitmap)
            for (layer in layers) {
                for (actor in layer.actors) {
                    var actorBitmap = BitmapFactory.decodeFile(actor.resourcePath)
                    actorBitmap = Bitmap.createScaledBitmap(actorBitmap, actor.sideLength, actor.sideLength, true)

                    canvas.save()
                    canvas.translate(trackX / 2f.pow(layers.indexOf(layer)), 0f)
                    canvas.drawBitmap(actorBitmap, actor.positionX.toFloat(), actor.positionY.toFloat(), Paint())
                    canvas.restore()

                    actorBitmap.recycle()
                }
            }
        }
        return sceneBitmap
    }
}